@@ -9,20 +9,21 @@ complexities of worker implementations, scopes, and treeherder annotations.
 """
 
 
+import functools
 import hashlib
 import os
 import re
 import time
 from copy import deepcopy
+from dataclasses import dataclass
+from typing import Callable
 
-import attr
 from voluptuous import All, Any, Extra, NotIn, Optional, Required
 
 from taskgraph import MAX_DEPENDENCIES
 from taskgraph.transforms.base import TransformSequence
 from taskgraph.util.hash import hash_path
 from taskgraph.util.keyed_by import evaluate_keyed_by
-from taskgraph.util.memoize import memoize
 from taskgraph.util.schema import (
     OptimizationSchema,
     Schema,
@@ -31,7 +32,7 @@ from taskgraph.util.schema import (
     taskref_or_string,
     validate_schema,
 )
-from taskgraph.util.treeherder import split_symbol
+from taskgraph.util.treeherder import split_symbol, treeherder_defaults
 from taskgraph.util.workertypes import worker_type_implementation
 
 from ..util import docker as dockerutil
@@ -42,7 +43,7 @@ RUN_TASK = os.path.join(
 )
 
 
-@memoize
+@functools.lru_cache(maxsize=None)
 def _run_task_suffix():
     """String to append to cache names under control of run-task."""
     return hash_path(RUN_TASK)[0:20]
@@ -71,6 +72,15 @@ task_description_schema = Schema(
                 ),
             ): object,
         },
+        Optional("priority"): Any(
+            "highest",
+            "very-high",
+            "high",
+            "medium",
+            "low",
+            "very-low",
+            "lowest",
+        ),
         # Soft dependencies of this task, as a list of tasks labels
         Optional("soft-dependencies"): [str],
         # Dependencies that must be scheduled in order for this task to run.
@@ -95,26 +105,40 @@ task_description_schema = Schema(
         Optional("extra"): {str: object},
         # treeherder-related information; see
         # https://schemas.taskcluster.net/taskcluster-treeherder/v1/task-treeherder-config.json
+        # This may be provided in one of two ways:
+        # 1) A simple `true` will cause taskgraph to generate the required information
+        # 2) A dictionary with one or more of the required keys. Any key not present
+        #    will use a default as described below.
         # If not specified, no treeherder extra information or routes will be
         # added to the task
-        Optional("treeherder"): {
-            # either a bare symbol, or "grp(sym)".
-            "symbol": str,
-            # the job kind
-            "kind": Any("build", "test", "other"),
-            # tier for this task
-            "tier": int,
-            # task platform, in the form platform/collection, used to set
-            # treeherder.machine.platform and treeherder.collection or
-            # treeherder.labels
-            "platform": str,
-        },
+        Optional("treeherder"): Any(
+            True,
+            {
+                # either a bare symbol, or "grp(sym)".
+                # The default symbol is the uppercased first letter of each
+                # section of the kind (delimited by "-") all smooshed together.
+                # Eg: "test" becomes "T", "docker-image" becomes "DI", etc.
+                "symbol": Optional(str),
+                # the task kind
+                # If "build" or "test" is found in the kind name, this defaults
+                # to the appropriate value. Otherwise, defaults to "other"
+                "kind": Optional(Any("build", "test", "other")),
+                # tier for this task
+                # Defaults to 1
+                "tier": Optional(int),
+                # task platform, in the form platform/collection, used to set
+                # treeherder.machine.platform and treeherder.collection or
+                # treeherder.labels
+                # Defaults to "default/opt"
+                "platform": Optional(str),
+            },
+        ),
         # information for indexing this build so its artifacts can be discovered;
         # if omitted, the build will not be indexed.
         Optional("index"): {
             # the name of the product this build produces
             "product": str,
-            # the names to use for this job in the TaskCluster index
+            # the names to use for this task in the TaskCluster index
             "job-name": str,
             # Type of gecko v2 index to use
             "type": str,
@@ -142,6 +166,15 @@ task_description_schema = Schema(
         Optional("run-on-projects"): optionally_keyed_by("build-platform", [str]),
         Optional("run-on-tasks-for"): [str],
         Optional("run-on-git-branches"): [str],
+        # The `shipping_phase` attribute, defaulting to None. This specifies the
+        # release promotion phase that this task belongs to.
+        Optional("shipping-phase"): Any(
+            None,
+            "build",
+            "promote",
+            "push",
+            "ship",
+        ),
         # The `always-target` attribute will cause the task to be included in the
         # target_task_graph regardless of filtering. Tasks included in this manner
         # will be candidates for optimization even when `optimize_target_tasks` is
@@ -155,7 +188,7 @@ task_description_schema = Schema(
         # be substituted in this string:
         #  {level} -- the scm level of this push
         "worker-type": str,
-        # Whether the job should use sccache compiler caching.
+        # Whether the task should use sccache compiler caching.
         Required("needs-sccache"): bool,
         # information specific to the worker implementation that will run this task
         Optional("worker"): {
@@ -172,7 +205,7 @@ TC_TREEHERDER_SCHEMA_URL = (
 
 
 UNKNOWN_GROUP_NAME = (
-    "Treeherder group {} (from {}) has no name; " "add it to taskcluster/ci/config.yml"
+    "Treeherder group {} (from {}) has no name; " "add it to taskcluster/config.yml"
 )
 
 V2_ROUTE_TEMPLATES = [
@@ -190,21 +223,28 @@ def get_branch_rev(config):
     return config.params["head_rev"]
 
 
-@memoize
+@functools.lru_cache(maxsize=None)
 def get_default_priority(graph_config, project):
     return evaluate_keyed_by(
         graph_config["task-priority"], "Graph Config", {"project": project}
     )
 
 
+@functools.lru_cache(maxsize=None)
+def get_default_deadline(graph_config, project):
+    return evaluate_keyed_by(
+        graph_config["task-deadline-after"], "Graph Config", {"project": project}
+    )
+
+
 # define a collection of payload builders, depending on the worker implementation
 payload_builders = {}
 
 
-@attr.s(frozen=True)
+@dataclass(frozen=True)
 class PayloadBuilder:
-    schema = attr.ib(type=Schema)
-    builder = attr.ib()
+    schema: Schema
+    builder: Callable
 
 
 def payload_builder(name, schema):
@@ -213,7 +253,8 @@ def payload_builder(name, schema):
     )
 
     def wrap(func):
-        payload_builders[name] = PayloadBuilder(schema, func)
+        assert name not in payload_builders, f"duplicate payload builder name {name}"
+        payload_builders[name] = PayloadBuilder(schema, func)  # type: ignore
         return func
 
     return wrap
@@ -225,6 +266,7 @@ index_builders = {}
 
 def index_builder(name):
     def wrap(func):
+        assert name not in index_builders, f"duplicate index builder name {name}"
         index_builders[name] = func
         return func
 
@@ -233,7 +275,7 @@ def index_builder(name):
 
 UNSUPPORTED_INDEX_PRODUCT_ERROR = """\
 The index product {product} is not in the list of configured products in
-`taskcluster/ci/config.yml'.
+`taskcluster/config.yml'.
 """
 
 
@@ -284,7 +326,7 @@ def verify_index(config, index):
             {
                 # only one type is supported by any of the workers right now
                 "type": "persistent",
-                # name of the cache, allowing re-use by subsequent tasks naming the
+                # name of the cache, allowing reuse by subsequent tasks naming the
                 # same cache
                 "name": str,
                 # location in the task image where the cache will be mounted
@@ -331,6 +373,9 @@ def build_docker_worker_payload(config, task, task_def):
         if "in-tree" in image:
             name = image["in-tree"]
             docker_image_task = "build-docker-image-" + image["in-tree"]
+            assert "docker-image" not in task.get(
+                "dependencies", ()
+            ), "docker-image key in dependencies object is reserved"
             task.setdefault("dependencies", {})["docker-image"] = docker_image_task
 
             image = {
@@ -344,10 +389,10 @@ def build_docker_worker_payload(config, task, task_def):
             for v in sorted(volumes):
                 if v in worker["volumes"]:
                     raise Exception(
-                        "volume %s already defined; "
+                        f"volume {v} already defined; "
                         "if it is defined in a Dockerfile, "
                         "it does not need to be specified in the "
-                        "worker definition" % v
+                        "worker definition"
                     )
 
                 worker["volumes"].append(v)
@@ -454,19 +499,19 @@ def build_docker_worker_payload(config, task, task_def):
 
         # run-task knows how to validate caches.
         #
-        # To help ensure new run-task features and bug fixes don't interfere
-        # with existing caches, we seed the hash of run-task into cache names.
-        # So, any time run-task changes, we should get a fresh set of caches.
-        # This means run-task can make changes to cache interaction at any time
-        # without regards for backwards or future compatibility.
+        # To help ensure new run-task features and bug fixes, as well as the
+        # versions of tools such as mercurial or git, don't interfere with
+        # existing caches, we seed the underlying docker-image task id into
+        # cache names, for tasks using in-tree Docker images.
         #
         # But this mechanism only works for in-tree Docker images that are built
         # with the current run-task! For out-of-tree Docker images, we have no
         # way of knowing their content of run-task. So, in addition to varying
         # cache names by the contents of run-task, we also take the Docker image
-        # name into consideration. This means that different Docker images will
-        # never share the same cache. This is a bit unfortunate. But it is the
-        # safest thing to do. Fortunately, most images are defined in-tree.
+        # name into consideration.
+        #
+        # This means that different Docker images will never share the same
+        # cache.  This is a bit unfortunate, but is the safest thing to do.
         #
         # For out-of-tree Docker images, we don't strictly need to incorporate
         # the run-task content into the cache name. However, doing so preserves
@@ -487,6 +532,8 @@ def build_docker_worker_payload(config, task, task_def):
                     out_of_tree_image.encode("utf-8")
                 ).hexdigest()
                 suffix += name_hash[0:12]
+            else:
+                suffix += "-<docker-image>"
 
         else:
             suffix = cache_version
@@ -506,19 +553,19 @@ def build_docker_worker_payload(config, task, task_def):
                 suffix=suffix,
             )
             caches[name] = cache["mount-point"]
-            task_def["scopes"].append("docker-worker:cache:%s" % name)
+            task_def["scopes"].append({"task-reference": f"docker-worker:cache:{name}"})
 
         # Assertion: only run-task is interested in this.
         if run_task:
             payload["env"]["TASKCLUSTER_CACHES"] = ";".join(sorted(caches.values()))
 
-        payload["cache"] = caches
+        payload["cache"] = {"task-reference": caches}
 
     # And send down volumes information to run-task as well.
     if run_task and worker.get("volumes"):
         payload["env"]["TASKCLUSTER_VOLUMES"] = ";".join(sorted(worker["volumes"]))
 
-    if payload.get("cache") and skip_untrusted:
+    if payload.get("cache") and skip_untrusted:  # type: ignore
         payload["env"]["TASKCLUSTER_UNTRUSTED_CACHES"] = "1"
 
     if features:
@@ -593,6 +640,11 @@ def build_docker_worker_payload(config, task, task_def):
         Required("env"): {str: taskref_or_string},
         # the maximum time to run, in seconds
         Required("max-run-time"): int,
+        # the exit status code(s) that indicates the task should be retried
+        Optional("retry-exit-status"): [int],
+        # the exit status code(s) that indicates the caches used by the task
+        # should be purged
+        Optional("purge-caches-exit-status"): [int],
         # os user groups for test task workers
         Optional("os-groups"): [str],
         # feature for test task to run as administarotr
@@ -615,6 +667,8 @@ def build_generic_worker_payload(config, task, task_def):
     on_exit_status = {}
     if "retry-exit-status" in worker:
         on_exit_status["retry"] = worker["retry-exit-status"]
+    if "purge-caches-exit-status" in worker:
+        on_exit_status["purgeCaches"] = worker["purge-caches-exit-status"]
     if worker["os"] == "windows":
         on_exit_status.setdefault("retry", []).extend(
             [
@@ -712,7 +766,7 @@ def build_generic_worker_payload(config, task, task_def):
     schema={
         # the maximum time to run, in seconds
         Required("max-run-time"): int,
-        # locale key, if this is a locale beetmover job
+        # locale key, if this is a locale beetmover task
         Optional("locale"): str,
         Optional("partner-public"): bool,
         Required("release-properties"): {
@@ -908,6 +962,78 @@ def add_generic_index_routes(config, task):
     return task
 
 
+@transforms.add
+def process_treeherder_metadata(config, tasks):
+    for task in tasks:
+        routes = task.get("routes", [])
+        extra = task.get("extra", {})
+        task_th = task.get("treeherder")
+
+        if task_th:
+            # This `merged_th` object is just an intermediary that combines
+            # the defaults and whatever is in the task. Ultimately, the task
+            # transforms this data a bit in the `treeherder` object that is
+            # eventually set in the task.
+            merged_th = treeherder_defaults(config.kind, task["label"])
+            if isinstance(task_th, dict):
+                merged_th.update(task_th)
+
+            treeherder = extra.setdefault("treeherder", {})
+            extra.setdefault("treeherder-platform", merged_th["platform"])
+
+            machine_platform, collection = merged_th["platform"].split("/", 1)
+            treeherder["machine"] = {"platform": machine_platform}
+            treeherder["collection"] = {collection: True}
+
+            group_names = config.graph_config["treeherder"]["group-names"]
+            groupSymbol, symbol = split_symbol(merged_th["symbol"])
+            if groupSymbol != "?":
+                treeherder["groupSymbol"] = groupSymbol
+                if groupSymbol not in group_names:
+                    path = os.path.join(config.path, task.get("task-from", ""))
+                    raise Exception(UNKNOWN_GROUP_NAME.format(groupSymbol, path))
+                treeherder["groupName"] = group_names[groupSymbol]
+            treeherder["symbol"] = symbol
+            if len(symbol) > 25 or len(groupSymbol) > 25:
+                raise RuntimeError(
+                    "Treeherder group and symbol names must not be longer than "
+                    "25 characters: {} (see {})".format(
+                        treeherder["symbol"],
+                        TC_TREEHERDER_SCHEMA_URL,
+                    )
+                )
+            treeherder["jobKind"] = merged_th["kind"]
+            treeherder["tier"] = merged_th["tier"]
+
+            branch_rev = get_branch_rev(config)
+
+            if config.params["tasks_for"].startswith("github-pull-request"):
+                # In the past we used `project` for this, but that ends up being
+                # set to the repository name of the _head_ repo, which is not correct
+                # (and causes scope issues) if it doesn't match the name of the
+                # base repo
+                base_project = config.params["base_repository"].split("/")[-1]
+                if base_project.endswith(".git"):
+                    base_project = base_project[:-4]
+                th_project_suffix = "-pr"
+            else:
+                base_project = config.params["project"]
+                th_project_suffix = ""
+
+            routes.append(
+                "{}.v2.{}.{}.{}".format(
+                    TREEHERDER_ROUTE_ROOT,
+                    base_project + th_project_suffix,
+                    branch_rev,
+                    config.params["pushlog_id"],
+                )
+            )
+
+        task["routes"] = routes
+        task["extra"] = extra
+        yield task
+
+
 @transforms.add
 def add_index_routes(config, tasks):
     for task in tasks:
@@ -920,7 +1046,7 @@ def add_index_routes(config, tasks):
         if rank == "by-tier":
             # rank is zero for non-tier-1 tasks and based on pushid for others;
             # this sorts tier-{2,3} builds below tier-1 in the index
-            tier = task.get("treeherder", {}).get("tier", 3)
+            tier = task.get("extra", {}).get("treeherder", {}).get("tier", 3)
             extra_index["rank"] = 0 if tier > 1 else int(config.params["build_date"])
         elif rank == "build_date":
             extra_index["rank"] = int(config.params["build_date"])
@@ -961,64 +1087,21 @@ def build_task(config, tasks):
         # set up extra
         extra = task.get("extra", {})
         extra["parent"] = os.environ.get("TASK_ID", "")
-        task_th = task.get("treeherder")
-        if task_th:
-            extra.setdefault("treeherder-platform", task_th["platform"])
-            treeherder = extra.setdefault("treeherder", {})
-
-            machine_platform, collection = task_th["platform"].split("/", 1)
-            treeherder["machine"] = {"platform": machine_platform}
-            treeherder["collection"] = {collection: True}
-
-            group_names = config.graph_config["treeherder"]["group-names"]
-            groupSymbol, symbol = split_symbol(task_th["symbol"])
-            if groupSymbol != "?":
-                treeherder["groupSymbol"] = groupSymbol
-                if groupSymbol not in group_names:
-                    path = os.path.join(config.path, task.get("task-from", ""))
-                    raise Exception(UNKNOWN_GROUP_NAME.format(groupSymbol, path))
-                treeherder["groupName"] = group_names[groupSymbol]
-            treeherder["symbol"] = symbol
-            if len(symbol) > 25 or len(groupSymbol) > 25:
-                raise RuntimeError(
-                    "Treeherder group and symbol names must not be longer than "
-                    "25 characters: {} (see {})".format(
-                        task_th["symbol"],
-                        TC_TREEHERDER_SCHEMA_URL,
-                    )
-                )
-            treeherder["jobKind"] = task_th["kind"]
-            treeherder["tier"] = task_th["tier"]
-
-            branch_rev = get_branch_rev(config)
-
-            if config.params["tasks_for"].startswith("github-pull-request"):
-                # In the past we used `project` for this, but that ends up being
-                # set to the repository name of the _head_ repo, which is not correct
-                # (and causes scope issues) if it doesn't match the name of the
-                # base repo
-                base_project = config.params["base_repository"].split("/")[-1]
-                if base_project.endswith(".git"):
-                    base_project = base_project[:-4]
-                th_project_suffix = "-pr"
-            else:
-                base_project = config.params["project"]
-                th_project_suffix = ""
-
-            routes.append(
-                "{}.v2.{}.{}.{}".format(
-                    TREEHERDER_ROUTE_ROOT,
-                    base_project + th_project_suffix,
-                    branch_rev,
-                    config.params["pushlog_id"],
-                )
-            )
 
         if "expires-after" not in task:
-            task["expires-after"] = "28 days" if config.params.is_try() else "1 year"
+            task["expires-after"] = (
+                config.graph_config._config.get("task-expires-after", "28 days")
+                if config.params.is_try()
+                else "1 year"
+            )
 
         if "deadline-after" not in task:
-            task["deadline-after"] = "1 day"
+            if "task-deadline-after" in config.graph_config:
+                task["deadline-after"] = get_default_deadline(
+                    config.graph_config, config.params["project"]
+                )
+            else:
+                task["deadline-after"] = "1 day"
 
         if "priority" not in task:
             task["priority"] = get_default_priority(
@@ -1056,16 +1139,30 @@ def build_task(config, tasks):
         if task.get("requires", None):
             task_def["requires"] = task["requires"]
 
-        if task_th:
+        if task.get("extra", {}).get("treeherder"):
+            branch_rev = get_branch_rev(config)
+            if config.params["tasks_for"].startswith("github-pull-request"):
+                # In the past we used `project` for this, but that ends up being
+                # set to the repository name of the _head_ repo, which is not correct
+                # (and causes scope issues) if it doesn't match the name of the
+                # base repo
+                base_project = config.params["base_repository"].split("/")[-1]
+                if base_project.endswith(".git"):
+                    base_project = base_project[:-4]
+                th_project_suffix = "-pr"
+            else:
+                base_project = config.params["project"]
+                th_project_suffix = ""
+
             # link back to treeherder in description
             th_push_link = (
                 "https://treeherder.mozilla.org/#/jobs?repo={}&revision={}".format(
                     config.params["project"] + th_project_suffix, branch_rev
                 )
             )
-            task_def["metadata"]["description"] += " ([Treeherder push]({}))".format(
-                th_push_link
-            )
+            task_def["metadata"][
+                "description"
+            ] += f" ([Treeherder push]({th_push_link}))"
 
         # add the payload and adjust anything else as required (e.g., scopes)
         payload_builders[task["worker"]["implementation"]].builder(
@@ -1089,6 +1186,14 @@ def build_task(config, tasks):
             attributes["run_on_git_branches"] = task["run-on-git-branches"]
 
         attributes["always_target"] = task["always-target"]
+        # This logic is here since downstream tasks don't always match their
+        # upstream dependency's shipping_phase.
+        # A text_type task['shipping-phase'] takes precedence, then
+        # an existing attributes['shipping_phase'], then fall back to None.
+        if task.get("shipping-phase") is not None:
+            attributes["shipping_phase"] = task["shipping-phase"]
+        else:
+            attributes.setdefault("shipping_phase", None)
 
         # Set MOZ_AUTOMATION on all jobs.
         if task["worker"]["implementation"] in (
@@ -1179,12 +1284,17 @@ def check_task_identifiers(config, tasks):
 def check_task_dependencies(config, tasks):
     """Ensures that tasks don't have more than 100 dependencies."""
     for task in tasks:
-        if len(task["dependencies"]) > MAX_DEPENDENCIES:
+        number_of_dependencies = (
+            len(task["dependencies"])
+            + len(task["if-dependencies"])
+            + len(task["soft-dependencies"])
+        )
+        if number_of_dependencies > MAX_DEPENDENCIES:
             raise Exception(
                 "task {}/{} has too many dependencies ({} > {})".format(
                     config.kind,
                     task["label"],
-                    len(task["dependencies"]),
+                    number_of_dependencies,
                     MAX_DEPENDENCIES,
                 )
             )
@@ -1196,7 +1306,7 @@ def check_caches_are_volumes(task):
 
     Caches and volumes are the only filesystem locations whose content
     isn't defined by the Docker image itself. Some caches are optional
-    depending on the job environment. We want paths that are potentially
+    depending on the task environment. We want paths that are potentially
     caches to have as similar behavior regardless of whether a cache is
     used. To help enforce this, we require that all paths used as caches
     to be declared as Docker volumes. This check won't catch all offenders.
@@ -1210,10 +1320,11 @@ def check_caches_are_volumes(task):
         return
 
     raise Exception(
-        "task %s (image %s) has caches that are not declared as "
-        "Docker volumes: %s "
-        "(have you added them as VOLUMEs in the Dockerfile?)"
-        % (task["label"], task["worker"]["docker-image"], ", ".join(sorted(missing)))
+        "task {} (image {}) has caches that are not declared as "
+        "Docker volumes: {} "
+        "(have you added them as VOLUMEs in the Dockerfile?)".format(
+            task["label"], task["worker"]["docker-image"], ", ".join(sorted(missing))
+        )
     )
 
 
@@ -1250,7 +1361,9 @@ def check_run_task_caches(config, tasks):
         main_command = command[0] if isinstance(command[0], str) else ""
         run_task = main_command.endswith("run-task")
 
-        for cache in payload.get("cache", {}):
+        for cache in payload.get("cache", {}).get(
+            "task-reference", payload.get("cache", {})
+        ):
             if not cache.startswith(cache_prefix):
                 raise Exception(
                     "{} is using a cache ({}) which is not appropriate "
@@ -1266,17 +1379,17 @@ def check_run_task_caches(config, tasks):
 
             if not run_task:
                 raise Exception(
-                    "%s is using a cache (%s) reserved for run-task "
+                    f"{task['label']} is using a cache ({cache}) reserved for run-task "
                     "change the task to use run-task or use a different "
-                    "cache name" % (task["label"], cache)
+                    "cache name"
                 )
 
-            if not cache.endswith(suffix):
+            if suffix not in cache:
                 raise Exception(
-                    "%s is using a cache (%s) reserved for run-task "
+                    f"{task['label']} is using a cache ({cache}) reserved for run-task "
                     "but the cache name is not dependent on the contents "
                     "of run-task; change the cache name to conform to the "
-                    "naming requirements" % (task["label"], cache)
+                    "naming requirements"
                 )
 
         yield task
