@@ -2,6 +2,7 @@
 # http://creativecommons.org/publicdomain/zero/1.0/
 
 import os
+import sys
 from pathlib import Path
 from textwrap import dedent
 
@@ -48,7 +49,7 @@ def run_taskgraph(maketgg, monkeypatch):
         ("morphed", ["_fake-t-0", "_fake-t-1"]),
     ),
 )
-def test_show_taskgraph(run_taskgraph, capsys, attr, expected):
+def test_show_taskgraph_attr(run_taskgraph, capsys, attr, expected):
     res = run_taskgraph([attr])
     assert res == 0
 
@@ -72,6 +73,34 @@ def test_show_taskgraph_parallel(run_taskgraph):
     assert res == 1
 
 
+def test_show_taskgraph_force_local_files_changed(mocker, run_taskgraph):
+    repo = mocker.MagicMock()
+    repo.get_changed_files.return_value = ["foo.txt"]
+
+    m = mocker.MagicMock()
+    m.get_repository.return_value = repo
+    mocker.patch.dict(sys.modules, {"taskgraph.util.vcs": m})
+
+    res = run_taskgraph(["full"])
+    assert res == 0
+    assert not repo.get_changed_files.called
+
+    res = run_taskgraph(["full", "--force-local-files-changed"])
+    assert res == 0
+    assert not repo.get_changed_files.called
+
+    res = run_taskgraph(
+        [
+            "full",
+            "--force-local-files-changed",
+            "-p",
+            "taskcluster/test/params/mc-onpush.yml",
+        ]
+    )
+    assert res == 0
+    assert repo.get_changed_files.call_count == 1
+
+
 def test_tasks_regex(run_taskgraph, capsys):
     run_taskgraph(["full", "--tasks=_.*-t-1"])
     out, _ = capsys.readouterr()
@@ -115,7 +144,7 @@ def test_output_file(run_taskgraph, tmpdir):
                     "description": "",
                     "kind": "task",
                     "label": "b",
-                    "optimization": None,
+                    "optimization": {"skip-unless-changed": True},
                     "soft_dependencies": [],
                     "if_dependencies": [],
                     "task": {
@@ -135,7 +164,7 @@ def test_output_file(run_taskgraph, tmpdir):
                     "description": "",
                     "kind": "task",
                     "label": "b",
-                    "optimization": None,
+                    "optimization": {"skip-unless-changed": True},
                     "soft_dependencies": [],
                     "if_dependencies": [],
                     "task": {
@@ -149,6 +178,7 @@ def test_output_file(run_taskgraph, tmpdir):
             None,
             [
                 "attributes.thing",
+                "optimization.skip-unless-changed",
                 "task.foo.baz",
             ],
             {
@@ -171,7 +201,7 @@ def test_output_file(run_taskgraph, tmpdir):
                     "description": "",
                     "kind": "task",
                     "label": "b",
-                    "optimization": None,
+                    "optimization": {},
                     "soft_dependencies": [],
                     "if_dependencies": [],
                     "task": {
@@ -187,7 +217,11 @@ def test_get_filtered_taskgraph(regex, exclude, expected):
     tasks = {
         "a": Task(kind="task", label="a", attributes={}, task={"foo": {"bar": 1}}),
         "b": Task(
-            kind="task", label="b", attributes={"thing": True}, task={"foo": {"baz": 1}}
+            kind="task",
+            label="b",
+            attributes={"thing": True},
+            optimization={"skip-unless-changed": True},
+            task={"foo": {"baz": 1}},
         ),
     }
 
